package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.user.dto.UserTagDTO;
import cn.rankin.data.api.user.dto.UserTagSearchDTO;
import cn.rankin.data.api.user.entity.UserTag;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
import cn.rankin.userservice.service.UserTagProductRelationService;
import cn.rankin.userservice.service.UserTagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping(value = "userTag")
public class UserTagController {

    @Autowired
    private UserTagService userTagService;

    @Autowired
    private UserTagProductRelationService userTagProductRelationService;

/*    @Autowired
    private MerchantProductRepository merchantProductRepository;*/

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public UserTag getUserTag(@PathVariable("id") String id) {

        UserTag userTag = userTagService.getUserTag(id);

        if(null != userTag){
            String userTagId = userTag.getId();
            List<UserTagProductRelation> userTagProductRelations = userTagProductRelationService.findByTagId(userTagId);
            userTag.setRelationList(userTagProductRelations);
        }

        return userTag;
    }

    @RequestMapping(value = "/code/{code}", method = RequestMethod.GET)
    public List<UserTag> findUserTagByCodeUid(@PathVariable("code") String code, @RequestParam("uid") String uid) {
        List<UserTag> userTagList = userTagService.findUserTagByCodeUid(code, uid);

        //标签下   课程数据
        userTagList.forEach(userTag -> {
            String userTagId = userTag.getId();
            List<UserTagProductRelation> userTagProductRelations = userTagProductRelationService.findByTagId(userTagId);
            userTag.setRelationList(userTagProductRelations);
        });

        return userTagList;
    }

    @RequestMapping(value = "/uid/{uid}", method = RequestMethod.GET)
    public List<UserTag> findUserTagByUid(@PathVariable("uid") String uid) {
        List<UserTag> userTagList = userTagService.findUserTagByUid(uid);

        //标签下   课程数据
        userTagList.forEach(userTag -> {
            String userTagId = userTag.getId();
            List<UserTagProductRelation> userTagProductRelations = userTagProductRelationService.findByTagId(userTagId);
            userTag.setRelationList(userTagProductRelations);
        });

        return userTagList;
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Page<UserTag> search(UserTagSearchDTO searchDTO) {
        UserTag userTag = new UserTag();

        String name = searchDTO.getName();
        if (name != null) {
            userTag.setName("%" + name + "%");
        }
        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        return userTagService.search(userTag, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(method = RequestMethod.POST)
    public UserTag create(@RequestBody UserTagDTO userTagDTO) {
        UserTag userTag = userTagService.create(userTagDTO);
        if(null == userTag){
            return null;
        }

        String userTagId = userTag.getId();
        List<String> productIdList = userTagDTO.getProductList();
        userTagProductRelationService.updateByTagId(userTagId,productIdList);

       /* List<MerchantProduct> merchantProductList = getMerchantProductList(userTagId);
        userTag.setProductList(merchantProductList);*/
        return userTag;
    }

    @RequestMapping(method = RequestMethod.PUT)
    public UserTag update(@RequestBody UserTagDTO userTagDTO) {
        UserTag result = userTagService.update(userTagDTO);
        if(null == result){
            return null;
        }

        String userTagId = result.getId();
        List<String> productIdList = userTagDTO.getProductList();
        userTagProductRelationService.updateByTagId(userTagId,productIdList);

        /*List<MerchantProduct> merchantProductList = getMerchantProductList(userTagId);
        result.setProductList(merchantProductList);*/

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public boolean delete(@PathVariable("id") String id) {
        return userTagService.delete(id);
    }


/*    public List<MerchantProduct> getMerchantProductList(String tagId) {
        List<UserTagProductRelation> relationList = userTagProductRelationService.findByTagId(tagId, BaseStatusEnum.NORMAL);
        if (CollectionUtils.isEmpty(relationList)) {
            return new ArrayList<>();
        }

        List<String> productIdList = new ArrayList<>();
        relationList.forEach(relation -> productIdList.add(relation.getPid()));
        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidIn(productIdList);
        Map<String, MerchantProduct> merchantProductMap = ListUtil.convert(merchantProductList, "pid", MerchantProduct.class);

        List<MerchantProduct> sortMerchantProductList = new ArrayList<>();
        for (String productId : productIdList) {
            MerchantProduct merchantProduct = merchantProductMap.get(productId);
            if (merchantProduct != null) {
                sortMerchantProductList.add(merchantProduct);
            }
        }
        return sortMerchantProductList;
    }*/

}
