package cn.rankin.userservice.service;

import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.MerchantProductTagRelation;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
//import cn.rankin.productservice.repository.MerchantProductTagRelationRepository;
import cn.rankin.userservice.repository.UserTagProductRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

@Slf4j
@Service
public class UserTagProductRelationService {

    @Autowired
    private UserTagProductRelationRepository relationRepository;

    public List<UserTagProductRelation> findByTagId(String tagId) {
        UserTagProductRelation relation = new UserTagProductRelation();
        relation.setTagId(tagId);
        relation.setStatus(BaseStatusEnum.NORMAL);


        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("sort", BaseOrderEnum.ASC);
        }};
        List<UserTagProductRelation> relationList = relationRepository.find(relation, JpaSortUtil.sort(sort));
/*
        if (CollectionUtils.isEmpty(relationList)) {
            return relationList;
        }

        List<String> productIdList = new ArrayList<>();

        relationList.forEach(userTagProductRelation -> {
            String productId = userTagProductRelation.getPid();
            productIdList.add(productId);
        });
*/

        return relationList;
    }

/*    public List<UserTagProductRelation> findByTagId(String tagId) {
        return relationRepository.findByTagId(tagId);
    }*/

    public List<UserTagProductRelation> findByTagId(String tagId, BaseStatusEnum status) {
        return relationRepository.findByTagId(tagId, status);
    }

    public List<UserTagProductRelation> findByPidsAndTagIds(List<String> pids, List<String> tagIds) {
        List<UserTagProductRelation> userTagProductRelationList = relationRepository.findByPidsAndTagIds(pids, tagIds);
        return userTagProductRelationList;
    }

    public List<UserTagProductRelation> findByPidAndTagId(String pid, String tagId) {
        return relationRepository.findByPidAndTagId(pid, tagId, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public void updateByPid(String productId, String tagId, List<String> tagIdList) {
        if (tagIdList == null) {
            return;
        }else if (tagIdList.size() == 0) {
            relationRepository.deleteByPidAndTagId(productId, tagId);
            return;
        }

        List<UserTagProductRelation> relationList = relationRepository.findByPidAndTagId(productId, tagId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String tid = relation.getTagId();
            if (!tagIdList.contains(tid)) {
                relation.setStatus(BaseStatusEnum.DEL);
            }else {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }
            currentIdList.add(tid);
        });

        List<String> notExistTagIdList = ListUtil.subtract(tagIdList, currentIdList);
        if (notExistTagIdList != null && notExistTagIdList.size() > 0) {
            notExistTagIdList.forEach( tid -> {
                UserTagProductRelation relation = new UserTagProductRelation();
                relation.setPid(productId);
                relation.setTagId(tid);
                relation.setSort(0);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        relationRepository.save(relationList);
    }

    @Transactional
    public void updateByTagId(String tagId, List<String> productIdList) {
        if (productIdList == null) {
            return;
        }else if (productIdList.size() == 0) {
            relationRepository.deleteByTagId(tagId);
            return;
        }

        List<UserTagProductRelation> relationList = relationRepository.findByTagId(tagId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            if (productIdList.contains(productId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            currentIdList.add(productId);
        });

        List<String> notExistItemIdList = ListUtil.subtract(productIdList, currentIdList);
        if (notExistItemIdList != null && notExistItemIdList.size() > 0) {
            notExistItemIdList.forEach( productId -> {
                UserTagProductRelation relation = new UserTagProductRelation();
                relation.setPid(productId);
                relation.setTagId(tagId);
                relation.setSort(0);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            Integer index = productIdList.indexOf(productId);
            relation.setSort(index);
        });

        relationRepository.save(relationList);
    }
}
